@@ -88,7 +88,6 @@ class Band:
         self.new_n = new_n
         self.bandgap = bandgap
         self.printbg = printbg
-        self.bg = None
         self.eigenval = Eigenval(os.path.join(folder, 'EIGENVAL'))
         self.efermi = float(os.popen(f'grep E-fermi {os.path.join(folder, "OUTCAR")}').read().split()[2]) + shift_efermi
         self.poscar = Poscar.from_file(
@@ -127,6 +126,7 @@ class Band:
 
         self.wavecar = os.path.join(folder, 'WAVECAR')
         self.projected = projected
+
         self.forbitals = self._check_f_orb()
         self.unfold = unfold
 
@@ -188,38 +188,52 @@ class Band:
             'd': 2,
             'f': 3,
         }
+        if self.bandgap:
+            self.bg = self._get_bandgap()
+        else:
+            self.bg = None
 
 
         if projected:
             self.pre_loaded_projections = os.path.isfile(os.path.join(folder, 'projected_eigenvalues.npy'))
             self.projected_eigenvalues = self._load_projected_bands()
 
-    def _get_bandgap(self, eigenvalues):
-        if np.sum(np.diff(np.sign(eigenvalues[:,:,0])) != 0) == 0:
-            occupied = eigenvalues[np.where(eigenvalues[:,:,-1] > 1e-8)]
-            unoccupied = eigenvalues[np.where(eigenvalues[:,:,-1] < 1e-8)]
-
-            vbm = np.max(occupied[:,0])
-            cbm = np.min(unoccupied[:,0])
-
-            bg = cbm - vbm
-        else:
-            bg = 0
-
-        if self.printbg:
-            print(f'Bandgap = {np.round(bg, 3)} eV')
-
-        self.bg = bg
+    def _get_bandgap(self):
+        from vaspvis.utils import get_bandgap
+        self.bg = get_bandgap(
+            folder=self.folder,
+            printbg=self.printbg,
+        )
 
     def _check_f_orb(self):
+        f_elements = [
+            'La',
+            'Ac',
+            'Ce',
+            'Tb',
+            'Th',
+            'Pr',
+            'Dy',
+            'Pa',
+            'Nd',
+            'Ho',
+            'U',
+            'Pm',
+            'Er',
+            'Np',
+            'Sm',
+            'Tm',
+            'Pu',
+            'Eu',
+            'Yb',
+            'Am',
+            'Gd',
+            'Lu',
+        ]
         f = False
         for element in self.poscar.site_symbols:
-            if element != 'H':
-                E = Element(element)
-                orbitals = list(E.atomic_orbitals.keys())
-                for orb in orbitals:
-                    if 'f' in orb:
-                        f = True
+            if element in f_elements:
+                f = True
         
         return f
                     
@@ -248,18 +262,9 @@ class Band:
             if self.ispin and not self.lsorbit:
                 eigenvalues = band_data[:,:,[0,2]]
                 kpoints = band_data[0,:,4:]
-                if self.bandgap:
-                    eigenvalues_up = band_data[:,:,[0,1]]
-                    eigenvalues_down = band_data[:,:,[2,3]]
-                    eigenvalues_bg = np.vstack([eigenvalues_up, eigenvalues_down])
             else:
                 eigenvalues = band_data[:,:,0]
                 kpoints = band_data[0,:,2:]
-                if self.bandgap:
-                    eigenvalues_bg = band_data[:,:,[0,1]]
-            
-            if self.bandgap:
-                self._get_bandgap(eigenvalues=eigenvalues_bg)
         else:
             if len(self.eigenval.eigenvalues.keys()) > 1:
                 eigenvalues_up = np.transpose(self.eigenval.eigenvalues[Spin.up], axes=(1,0,2))
@@ -270,13 +275,9 @@ class Band:
                     [eigenvalues_up, eigenvalues_down],
                     axis=2
                 )
-                if self.bandgap:
-                    eigenvalues_bg = np.vstack([eigenvalues_up, eigenvalues_down])
             else:
                 eigenvalues = np.transpose(self.eigenval.eigenvalues[Spin.up], axes=(1,0,2))
                 eigenvalues[:,:,0] = eigenvalues[:,:,0] - self.efermi
-                if self.bandgap:
-                    eigenvalues_bg = eigenvalues
 
             kpoints = np.array(self.eigenval.kpoints)
 
@@ -284,13 +285,8 @@ class Band:
                 kpoint_weights = np.array(self.eigenval.kpoints_weights)
                 zero_weight = np.where(kpoint_weights == 0)[0]
                 eigenvalues = eigenvalues[:,zero_weight]
-                if self.bandgap:
-                    eigenvalues_bg = eigenvalues_bg[:, zero_weight]
                 kpoints = kpoints[zero_weight]
 
-            if self.bandgap:
-                self._get_bandgap(eigenvalues=eigenvalues_bg)
-
             band_data = np.append(
                 eigenvalues,
                 np.tile(kpoints, (eigenvalues.shape[0],1,1)),
@@ -641,14 +637,21 @@ class Band:
         kpath_obj = HighSymmKpath(structure)
         kpath_labels = np.array(list(kpath_obj._kpath['kpoints'].keys()))
         kpath_coords = np.array(list(kpath_obj._kpath['kpoints'].values()))
-        index = np.where((self.kpoints[:, None] == kpath_coords).all(-1).any(-1) == True)[0]
+        index = np.where(
+            np.isclose(
+                self.kpoints[:, None],
+                kpath_coords,
+            ).all(-1).any(-1) == True
+        )[0]
+        #  index = np.where(np.isclose(self.kpoints[:, None], kpath_coords).all(-1).any(-1) == True)[0]
+        #  index = np.where((self.kpoints[:, None] == kpath_coords).all(-1).any(-1) == True)[0]
         index = [index[0]] + [index[i] for i in range(1,len(index)-1) if i % 2] + [index[-1]]
         kpoints_in_band = self.kpoints[index]
 
         label_index = []
         for i in range(kpoints_in_band.shape[0]):
             for j in range(kpath_coords.shape[0]):
-                if (kpoints_in_band[i] == kpath_coords[j]).all():
+                if (np.round(kpoints_in_band[i], 5) == np.round(kpath_coords[j], 5)).all():
                     label_index.append(j)
 
         kpoints_index = index
@@ -660,7 +663,10 @@ class Band:
         for k in kpoints_index:
             ax.axvline(x=wave_vectors[k], color=vlinecolor, alpha=0.7, linewidth=0.5)
 
-        plt.xticks(kpoints_index, kpath)
+        plt.xticks(
+            [wave_vectors[k] for k in kpoints_index],
+            kpath
+        )
 
     def _get_kticks_unfold(self, ax, wave_vectors, vlinecolor):
         if type(self.kpath) == str:
@@ -698,9 +704,16 @@ class Band:
             structure = self.poscar.structure
             kpath_obj = HighSymmKpath(structure)
             kpath_coords = np.array(list(kpath_obj._kpath['kpoints'].values()))
-            index = np.where((self.kpoints[:, None] == kpath_coords).all(-1).any(-1) == True)[0]
-            index = [index[0]] + [index[i] for i in range(1,len(index)-1) if i % 2] + [index[-1]]
-            slices = [np.s_[index[i]:index[i+1]+1] for i in range(len(index)-1)]
+            #  index = np.where((self.kpoints[:, None] == kpath_coords).all(-1).any(-1) == True)[0]
+
+            index = np.where(
+                np.isclose(
+                    self.kpoints[:, None],
+                    kpath_coords,
+                ).all(-1).any(-1) == True
+            )[0]
+            index = [index[0]-1] + [index[i] for i in range(1,len(index)-1) if i % 2] + [index[-1]]
+            slices = [np.s_[index[i]+1:index[i+1]+1] for i in range(len(index)-1)]
 
         if unfold and not hse:
             index = [0] + [(self.n * i) for i in range(1, len(self.kpath))]
@@ -710,9 +723,19 @@ class Band:
 
     def _get_interpolated_data(self, wave_vectors, data, crop_zero=False, kind='cubic'):
         slices = self._get_slices(unfold=self.unfold, hse=self.hse)
-        data = [data[:,i] for i in slices]
+        data_shape = data.shape
+        if len(data_shape) == 1:
+            data = [data[i] for i in slices]
+        else:
+            data = [data[:,i] for i in slices]
+
         wave_vectors = [wave_vectors[i] for i in slices]
-        fs = [interp1d(i, j, kind=kind, axis=1) for (i,j) in zip(wave_vectors, data)]
+
+        if len(data_shape) == 1:
+            fs = [interp1d(i, j, kind=kind, axis=0) for (i,j) in zip(wave_vectors, data)]
+        else:
+            fs = [interp1d(i, j, kind=kind, axis=1) for (i,j) in zip(wave_vectors, data)]
+
         new_wave_vectors = [np.linspace(wv.min(), wv.max(), self.new_n) for wv in wave_vectors]
         data = np.hstack([f(wv) for (f, wv) in zip(fs, new_wave_vectors)])
         wave_vectors = np.hstack(new_wave_vectors)
@@ -846,6 +869,9 @@ class Band:
         powernorm=False,
         gamma=0.5,
         projection=None,
+        highlight_band=False,
+        highlight_band_color='red',
+        band_index=None,
     ):
         """
         This function plots a plain band structure.
@@ -858,18 +884,36 @@ class Band:
         """
         bands_in_plot = self._filter_bands(erange=erange)
         eigenvalues = self.eigenvalues[bands_in_plot]
+
+        if highlight_band:
+            if band_index is not None:
+                highlight_eigenvalues = self.eigenvalues[int(band_index)]
+
         wave_vectors = self._get_k_distance()
         wave_vectors_for_kpoints = wave_vectors
 
         if self.interpolate:
             wave_vectors, eigenvalues = self._get_interpolated_data(wave_vectors_for_kpoints, eigenvalues)
 
+            if highlight_band:
+                if band_index is not None:
+                    _, highlight_eigenvalues = self._get_interpolated_data(
+                        wave_vectors_for_kpoints,
+                        highlight_eigenvalues,
+                    )
+            
+
+
         eigenvalues_ravel = np.ravel(np.c_[eigenvalues, np.empty(eigenvalues.shape[0]) * np.nan])
         wave_vectors_tile = np.tile(np.append(wave_vectors, np.nan), eigenvalues.shape[0])
 
         if self.unfold:
             spectral_weights = self.spectral_weights[bands_in_plot]
-            spectral_weights = spectral_weights / np.max(spectral_weights)
+            #  spectral_weights = spectral_weights / np.max(spectral_weights)
+
+            if highlight_band:
+                if band_index is not None:
+                    highlight_spectral_weights = self.spectral_weights[int(band_index)]
 
             if self.interpolate:
                 _, spectral_weights = self._get_interpolated_data(
@@ -878,6 +922,15 @@ class Band:
                     crop_zero=True,
                     kind='linear',
                 )
+
+                if highlight_band:
+                    if band_index is not None:
+                        _, highlight_spectral_weights = self._get_interpolated_data(
+                            wave_vectors_for_kpoints,
+                            highlight_spectral_weights,
+                            crop_zero=True,
+                            kind='linear',
+                        )
             
             spectral_weights_ravel = np.ravel(np.c_[spectral_weights, np.empty(spectral_weights.shape[0]) * np.nan])
 
@@ -903,6 +956,16 @@ class Band:
                     s=scale_factor * spectral_weights_ravel,
                     zorder=0,
                 )
+                if highlight_band:
+                    if band_index is not None:
+                        ax.scatter(
+                            wave_vectors,
+                            highlight_eigenvalues,
+                            c=highlight_band_color,
+                            ec=None,
+                            s=scale_factor * highlight_spectral_weights,
+                            zorder=100,
+                        )
         else:
             if heatmap:
                 self._heatmap(
@@ -926,6 +989,16 @@ class Band:
                     linestyle=linestyle,
                     zorder=0,
                 )
+                if highlight_band:
+                    if band_index is not None:
+                        ax.plot(
+                            wave_vectors,
+                            highlight_eigenvalues,
+                            color=highlight_band_color,
+                            linewidth=linewidth,
+                            linestyle=linestyle,
+                            zorder=100,
+                        )
 
         if self.hse:
             self._get_kticks_hse(ax=ax, wave_vectors=wave_vectors_for_kpoints, kpath=self.kpath, vlinecolor=vlinecolor)
@@ -970,7 +1043,8 @@ class Band:
             band_color (string): Color of the plain band structure
         """
         if self.unfold:
-            band_color = [(0.9,0.9,0.9)]
+            if band_color == 'black':
+                band_color = 'darkgrey'
             scale_factor = scale_factor * 4
 
         bands_in_plot = self._filter_bands(erange=erange)
@@ -1162,7 +1236,7 @@ class Band:
         orbitals='spd',
         erange=[-6,6],
         display_order=None,
-        color_dict=None,
+        color_list=None,
         legend=True,
         linewidth=0.75,
         band_color='black',
@@ -1194,15 +1268,18 @@ class Band:
             linewidth (float): Line width of the plain band structure plotted in the background
             band_color (string): Color of the plain band structure
         """
-        if color_dict is None:
-            color_dict = {
-                0: self.color_dict[0],
-                1: self.color_dict[1],
-                2: self.color_dict[2],
-                3: self.color_dict[4],
-            }
-
-        colors = np.array([color_dict[self.spd_relations[i]] for i in orbitals])
+        if color_list is None:
+            color_list = [
+                self.color_dict[0],
+                self.color_dict[1],
+                self.color_dict[2],
+                self.color_dict[4]
+            ]
+            colors = np.array(
+                [color_list[i] for i in range(len(orbitals))]
+            )
+        else:
+            colors = color_list
 
         projected_data = self._sum_spd(spd=orbitals)
 
@@ -1665,12 +1742,11 @@ class Band:
 
 
 if __name__ == "__main__":
-    pass
-    #  M = [
-        #  [0,1,-1],
-        #  [1,-1,0],
-        #  [-14,-14,-14]
-    #  ]
+    M = [
+        [0,1,-1],
+        [1,-1,0],
+        [-14,-14,-14]
+    ]
 #
     #  high_symm_points = [
         #  [2/3, 1/3, 1/3],
@@ -1678,24 +1754,21 @@ if __name__ == "__main__":
         #  [2/3, 1/3, 1/3],
     #  ]
 #
-    #  #  high_symm_points = [
-        #  #  [0.1, 0.1, 0],
-        #  #  [0.0, 0.0, 0],
-        #  #  [0.1, 0.1, 0],
-    #  #  ]
-#
-    #  band = Band(
-        #  folder="../../vaspvis_data/bandMGM",
-        #  projected=False,
-        #  unfold=True,
-        #  kpath='AGA',
-        #  high_symm_points=high_symm_points,
-        #  n=40,
-        #  M=M,
-    #  )
-    #  fig, ax = plt.subplots(figsize=(3,4), dpi=300)
+    high_symm_points = [
+        [0.1, 0.1, 0],
+        [0.0, 0.0, 0],
+        [0.1, 0.1, 0],
+    ]
+
+    band = Band(
+        folder="/home/derek/for_James/band",
+        projected=False,
+    )
+    band.eigenvalues
+    fig, ax = plt.subplots(figsize=(3,4), dpi=300)
     #  start = time.time()
-    #  #  band.plot_plain(ax=ax, color=[(0.9,0.9,0.9)])
+    band.plot_plain(ax=ax, color=[(0.9,0.9,0.9)])
+    band._get_bandgap()
     #  #  band.plot_spd(ax=ax, orbitals='sd', display_order='all', scale_factor=35, erange=[-5,0])
     #  #  band.plot_orbitals(ax=ax, scale_factor=35, orbitals=range(8), display_order=None)
     #  band.plot_plain(
